package com.gic.haoban.manage.api.service;

import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.manage.api.dto.*;

import java.util.List;
import java.util.Set;

public interface WxEnterpriseApiService {

	/**
	 * 好办接收企业微信回调信息
	 *
	 * @param dto
	 */
	void callbackReviceEnterprise(QywxCorpInfoSimpleDTO dto);
	/**
	 * 运维后台获取微信企业id列表
	 * @return
	 */
	com.gic.api.base.commons.Page<YwWxEnterpriseDTO> list(BasePageInfo pageInfo,String keyword,List<String> enterpriseIds,Integer gicContactFlag,String version, List<String> enterpriseIdsNotIn);
	/**
	 * 获取一个
	 * @param wxEnterpriseId
	 * @return
	 */
	WxEnterpriseDTO getOne(String wxEnterpriseId);

	WxEnterpriseDTO getEnterpriseBycorpId(String corpId);
	/**
	 * 更新企业
	 * @param enterpriseDTO
	 */
	void update(WxEnterpriseDTO enterpriseDTO);
	
	List<YwWxEnterpriseDTO> listAll();
	
	List<YwWxEnterpriseDTO> listByIds(Set<String> enterpriseIds);
	
	boolean enterpriseIsOver(String enterpriseId);

	/**
	 * 获取配置
	 *
	 * @param wxEnterpriseId
	 * @param secretType
	 * @param enterpriseId
	 * @return
	 */
	SecretSettingDTO getSecretSetting(String wxEnterpriseId, int secretType, String enterpriseId);

	/**
	 * 保存
	 *
	 * @param secretSettingDTO
	 * @return
	 */
	boolean saveSecretSetting(SecretSettingDTO secretSettingDTO);

	/**
	 * secretList
	 * @param wxEnterpriseId
	 * @return
	 */
	List<SecretSettingDTO> listSecret(String wxEnterpriseId);

	/**
	 * 删除
	 * @param secretId
	 * @return
	 */
	boolean delSecretSetting(String secretId);

	/**
	 * 保存小程序配置
	 *
	 * @param miniprogramSettingDTO
	 * @return
	 */
	boolean saveMiniprogramSetting(MiniprogramSettingDTO miniprogramSettingDTO);


	/**
	 * 查询小程序配置列表
	 *
	 * @param wxEnterpriseId
	 * @return
	 */
	List<MiniprogramSettingDTO> listMiniprogramSetting(String wxEnterpriseId);

	/**
	 * 获取单个小程序配置
	 *
	 * @param wxEnterpriseId
	 * @param enterpriseId
	 * @return
	 */
	MiniprogramSettingDTO getMiniprogramSettingByEid(String wxEnterpriseId, String enterpriseId);


	/**
	 * 获取单个小程序配置
	 *
	 * @param wxEnterpriseId
	 * @param appId
	 * @return
	 */
	MiniprogramSettingDTO getMiniprogramSettingByAppId(String wxEnterpriseId, String appId);

	/**
	 * 删除 小程序配置
	 *
	 * @return
	 */
	boolean delMiniprogramSettingByEid(String id);

	/**
	 * @param dto
	 * @return
	 */
	boolean saveWelcome(WelcomeDTO dto);

	/**
	 * 获取欢迎语
	 *
	 * @param wxEnterpriseId
	 * @return
	 */
	WelcomeDTO getWelcome(String wxEnterpriseId);

	/**
	 * 获取欢迎语
	 *
	 * @param wxEnterpriseId
	 * @return
	 */
	boolean setWelcome(String wxEnterpriseId, Integer openFlag);

	/**
	 * 删除欢迎语
	 *
	 * @param wxEnterpriseId
	 * @return
	 */
	boolean delWelcome(String wxEnterpriseId);


	/**
	 * 企微回调-同意授权转换external_userid
	 *
	 * @param corpId        公司标识(同意授权的企业ID)
	 * @param serviceCorpId 服务公司id(服务商企业ID)
	 * @author mozhu
	 * @date 2021-11-30 19:14:08
	 */
	void agreeExternalUseridCallBack(String corpId, String serviceCorpId);


	/**
	 * 同意外部标识标志
	 *
	 * @param wxEnterpriseId 公司标识
	 * @param openFlag       开放的标志
	 * @author mozhu
	 * @date 2021-12-18 11:55:51
	 */
	void agreeExternalUseridFlag(String wxEnterpriseId, Integer openFlag);

	/**
	 * 好办登录
	 *
	 * @param staffLoginDTO 员工登录dto
	 * @author mozhu
	 * @date 2021-12-15 11:02:52
	 */
	void wellDoneLogin(StaffLoginDTO staffLoginDTO);
}
