package com.gic.haoban.manage.service.task.friend;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.MemberUnionidRelatedApiService;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.haoban.manage.service.task.BaseSyncOperation;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 微信好友
 */
@Service
public class FriendClerkSyncNewOperation implements BaseSyncOperation {

    private static final Logger logger = LoggerFactory.getLogger(FriendClerkSyncNewOperation.class);
    @Autowired
    private PreDealService preDealService;
    @Autowired
    private MemberUnionidRelatedApiService memberUnionidRelatedApiService;
    @Autowired
    private SyncTaskService syncTaskService;
    private static final String FRIEND_LOCK = "haoban_fresh_wx_friend_lock_";

    private static final String TRY_AGAIN = "haoban_fresh_wx_friend_again_";

    @Override
    public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        logger.info("dealParamMqDTO={},dataPre={}", JSON.toJSONString(dealParamMqDTO), JSON.toJSONString(dataPre));
        String reason = "";
        boolean dealFlag = true;
        try {
            String wxEnterpriseId = dataPre.getWxEnterpriseId();
            String wxUserId = dataPre.getDataId();
            //获取所有外部联系人的外部联系人ids
            List<String> selfUserIdList = memberUnionidRelatedApiService.listSelfExterialList(wxEnterpriseId, wxUserId);
            logger.info("selfUserIdList={}", JSONObject.toJSONString(selfUserIdList));
            if (CollectionUtils.isEmpty(selfUserIdList)) {
                logger.info("自建应用调用第三方接口进入重试");
                dealFlag = tryAgainToMq(dataPre);
                reason = "重试次数过多";
                return;
            }
            if (CollectionUtils.isEmpty(selfUserIdList)) {
                logger.info("改用户没有外部联系人：{}", JSONObject.toJSONString(dataPre));
                dealFlag = false;
                reason = "该用户没有外部联系人";
                return;
            }
            //第三方服务商的外部联系人
            List<String> userIdList = memberUnionidRelatedApiService.listExterialList(wxEnterpriseId, wxUserId);
            if (CollectionUtils.isEmpty(selfUserIdList)) {
                dealFlag = tryAgainToMq(dataPre);
                reason = "重试次数过多";
                logger.info("第三方服务商外部联系人调用失败进入重试：{}", dataPre.getDataId());
                return;
            }
            if (CollectionUtils.isEmpty(userIdList)) {
                logger.info("改用户没有外部联系人2：{}", JSONObject.toJSONString(dataPre));
                dealFlag = false;
                reason = "该用户没有外部联系人";
                return;
            }
            //清理外部联系人
            memberUnionidRelatedApiService.cleanNoExsitFriendNew(wxEnterpriseId, wxUserId, userIdList);
            //写入外部联系人的处理队列
            List<TabHaobanPreDealLog> dealLogList = selfUserIdList.stream().map(externalUserId -> {
                TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
                dealLog.setDataId(externalUserId + "-" + dataPre.getDataId());
                dealLog.setpDataId(dataPre.getDataId());
                dealLog.setDataType(PreDealTypeEnum.self_friend.getVal());
                dealLog.setStatusFlag(0);
                dealLog.setTaskId(dataPre.getTaskId());
                dealLog.setDataContent(externalUserId);
                dealLog.setWxEnterpriseId(wxEnterpriseId);
                return dealLog;
            }).collect(Collectors.toList());
            preDealService.insert(dealLogList);

            //写入外部联系人的处理队列
            List<TabHaobanPreDealLog> thirddealLogList = userIdList.stream().map(externalUserId -> {
                TabHaobanPreDealLog dealLog = new TabHaobanPreDealLog();
                dealLog.setDataId(externalUserId + "-" + dataPre.getDataId());
                dealLog.setpDataId(dataPre.getDataId());
                dealLog.setDataType(PreDealTypeEnum.friend.getVal());
                dealLog.setStatusFlag(0);
                dealLog.setTaskId(dataPre.getTaskId());
                dealLog.setDataContent(externalUserId);
                dealLog.setWxEnterpriseId(wxEnterpriseId);
                return dealLog;
            }).collect(Collectors.toList());
            preDealService.insert(thirddealLogList);
            //成功更新状态
            dealSuccess(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), dataPre.getWxEnterpriseId());
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("同步失败：{},{}", JSONObject.toJSONString(dataPre), e);
            reason = "成员好友处理异常";
            dealFlag = false;
        } finally {
            if (!dealFlag) {
                dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), reason);
            }
        }
    }

    /**
     * 重试机制
     *
     * @param dataPre
     * @return
     */
    private boolean tryAgainToMq(TabHaobanPreDealLog dataPre) {
        logger.info("需要重试：{}", dataPre.getDataId());
        String key = TRY_AGAIN + dataPre.getTaskId() + ":" + dataPre.getDataId();
        Object cache = RedisUtil.getCache(key);
        if (null == cache) {
            RedisUtil.setCache(key, 1, 2L, TimeUnit.HOURS);
        } else {
            Integer count = Integer.valueOf(cache.toString());
            RedisUtil.setCache(key, count + 1);
            if (count > 4) {
                return false;
            }
        }
        HashSet<String> reTrysIds = new HashSet<>();
        reTrysIds.add(dataPre.getDataId());
        dealDepartmentToMq(dataPre.getTaskId(), reTrysIds, SyncTaskStatusEnum.friend_clerk_sync);
        return true;
    }

    @Override
    public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
        preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);
        checkDepartmentTask(taskId);
    }

    @Override
    public void dealSuccess(String taskId, String dataId, String enterpriseId, String wxEnterpriseId) {
        preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.dealing.getVal(), "预处理中");
        List<String> dataIds = preDealService.listReDataIdByPDataId(taskId, dataId, PreDealTypeEnum.self_friend.getVal());
        logger.info("{}:处理的数据：{}", dataId, JSONObject.toJSONString(dataIds));
        dealDepartmentToMq(taskId, new HashSet<>(dataIds), SyncTaskStatusEnum.self_friend_sync);
        checkDepartmentTask(taskId);
    }

    @Override
    public void checkDepartmentTask(String taskId) {
        boolean b = preDealService.checkFriendTask(taskId, PreDealTypeEnum.friend_clerk.getVal());
        if (!b) {
            return;
        }
        logger.info("好友同步成功:{}", taskId);
        String key = "haoban_sync_friend_task_" + taskId;
        RedisUtil.lock(key, 3L);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        if (syncTask.getStatusFlag() == SyncTaskStatusEnum.compute.getVal()
                || syncTask.getStatusFlag() == SyncTaskStatusEnum.exception_compute.getVal()) {
            logger.info("已经处理成功:{}", taskId);
            return;
        }
        int errCount = preDealService.countByTaskId(taskId, -1, PreDealStatusEnum.exception.getVal());
        if (errCount > 0) {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_compute.getVal());
        } else {
            syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
        }
        String staffLockKey = FRIEND_LOCK + syncTask.getAddUser();
        RedisUtil.delCache(staffLockKey);
        RedisUtil.unlock(key);
    }


    /**
     * 放入mq处理部门数据
     *
     * @param taskId
     * @param dealList
     */
    private void dealDepartmentToMq(String taskId, Set<String> dealList, SyncTaskStatusEnum syncTaskStatusEnum) {
        //预处理分组任务
        syncTaskService.updateTaskStatus(taskId, syncTaskStatusEnum.getVal());
        List<String> ret = dealList.stream().map(relationId -> {
            DealParamMqDTO dealParamMqDTO = new DealParamMqDTO();
            dealParamMqDTO.setData(relationId);
            dealParamMqDTO.setTaskId(taskId);
            dealParamMqDTO.setType(syncTaskStatusEnum.getVal());
            return JSONObject.toJSONString(dealParamMqDTO);
        }).collect(Collectors.toList());

        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            clientInstance.sendBatchMessages("departmentSyncDealMq", ret);
        } catch (Exception e) {
            logger.info("发送失败:{},{}", taskId);
            e.printStackTrace();
        }
    }

}
