package com.gic.haoban.manage.web.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.dict.api.dto.DictDTO;
import com.gic.dict.api.service.ManagerDictService;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.app.customer.dto.StaffOpenRelatedDTO;
import com.gic.haoban.app.customer.service.api.service.StaffMemberRelationApiService;
import com.gic.haoban.common.anno.IgnoreLogin;
import com.gic.haoban.common.utils.AuthRequestWellDoneUtil;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.common.utils.JwtUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.GetUserByMemberCodeQo;
import com.gic.haoban.manage.web.qo.MemberLoginQo;
import com.gic.haoban.manage.web.qo.StaffLoginQO;
import com.gic.haoban.manage.web.utils.IPAddressUtil;
import com.gic.haoban.manage.web.vo.AppStaffVo;
import com.gic.haoban.manage.web.vo.MemberSendMessageVo;
import com.gic.haoban.manage.web.vo.StoreMemberVO;
import com.gic.haoban.manage.web.vo.WellDoneLoginInfoVO;
import com.gic.member.api.service.MemberOpenCardBusinessService;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxCorpApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

@RestController
public class WxEnterpriseInfoController extends WebBaseController {

    private static final Logger logger = LoggerFactory.getLogger(WxEnterpriseInfoController.class);

    @Autowired
    private QywxCorpApiService qywxCorpApiService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private com.gic.enterprise.api.service.StoreService storeService;
    @Autowired
    private MemberUnionidRelatedApiService memberUnionidRelatedApiService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private ManagerDictService managerDictService;
    @Autowired
    private StaffMemberRelationApiService staffMemberRelationApiService;
    @Autowired
    private StaffClerkRelationApiService staffClerkRelationApiService;
    @Autowired
    private ExternalClerkRelatedApiService externalClerkRelatedApiService;
    @Autowired
    private MemberOpenCardBusinessService memberOpenCardBusinessService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private Config config;
    @Autowired
    private WxApplicationApiService wxApplicationApiService;


    /**
     * code授权
     *
     * @param code
     * @return
     */
    @RequestMapping(value = "get-info-by-code", method = {RequestMethod.GET, RequestMethod.POST})
    @IgnoreLogin
    public HaobanResponse getInfoByCode(String code) {
        String userDTOJson = qywxCorpApiService.getCodeSession(config.getWxSuiteid(), code);
        logger.info("userDTOJson=============>{}", userDTOJson);
        UserDTO user = JSON.parseObject(userDTOJson, UserDTO.class);
        if (user == null) {
            return resultResponse(HaoBanErrCode.ERR_6);
        }
        String userId = user.getUserid();
        String corpId = user.getCorpid();
        WxEnterpriseDTO enterprise = wxEnterpriseApiService.getEnterpriseBycorpId(corpId);
        StaffDTO loginStaff = null;
        if (enterprise != null) {
            loginStaff = staffApiService.selectByUserIdAndEnterpriseId(userId, enterprise.getWxEnterpriseId());
        }
        if (loginStaff == null) {
            if (enterprise != null) {
                staffApiService.wxGetAdd(userId, enterprise.getWxEnterpriseId());
                loginStaff = staffApiService.selectByUserIdAndEnterpriseId(userId, enterprise.getWxEnterpriseId());
            }
        }
        if (loginStaff == null) {
            return resultResponse(HaoBanErrCode.ERR_600001);
        }
        if (StringUtils.isNotBlank(loginStaff.getPhoneNumber())) {
            DictDTO loginUserId = managerDictService.getSMSDict("app_login_phone_num", loginStaff.getPhoneNumber());
            logger.info("伪登录：{}，userId:{}", loginStaff.getPhoneNumber(), JSONObject.toJSONString(loginUserId));
            if (null != loginUserId) {
                String val = loginUserId.getDictValue();
                List<StaffDTO> staffDTOS = new ArrayList<>();
                List<StaffDTO> phoneDtos = staffApiService.listByPhoneNumber(val);
                List<StaffDTO> userDtos = staffApiService.listByWxUserId(val);
                if (CollectionUtils.isNotEmpty(phoneDtos)) {
                    staffDTOS.addAll(phoneDtos);
                }
                if (CollectionUtils.isNotEmpty(userDtos)) {
                    staffDTOS.addAll(userDtos);
                }
                if (CollectionUtils.isNotEmpty(staffDTOS)) {
                    if (staffDTOS.size() > 1) {
                        String dictName = loginUserId.getDictName();
                        loginStaff = staffDTOS.stream().filter(staffDTO -> dictName.indexOf(staffDTO.getWxEnterpriseId()) > 0).findFirst().orElse(loginStaff);
                    } else {
                        loginStaff = staffDTOS.get(0);
                    }
                }
            }
        }
        String staffId = loginStaff.getStaffId();
        //激活状态
        if (loginStaff.getActiveFlag() == 0) {
            staffApiService.activeStaff(staffId);
        }
        StaffPrivacyUseLogDTO staffPrivacyUseLogDTO = staffApiService.getStaffPrivacyByStaffId(staffId);

        String wxEnterpriseId = loginStaff.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
        String accesstoken = qywxCorpApiService.getCorpAccessToken(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid());
        WellDoneLoginInfoVO wellDoneLoginInfoVo = new WellDoneLoginInfoVO();
        wellDoneLoginInfoVo.setAccessToken(accesstoken);
        wellDoneLoginInfoVo.setStaffId(staffId);
        wellDoneLoginInfoVo.setWxEnterpriseId(wxEnterpriseId);
        wellDoneLoginInfoVo.setStaffName(loginStaff.getStaffName());
        wellDoneLoginInfoVo.setPhoneNumber(loginStaff.getPhoneNumber());
        wellDoneLoginInfoVo.setNationcode(loginStaff.getNationCode());
        wellDoneLoginInfoVo.setWxUserId(loginStaff.getWxUserId());
        wellDoneLoginInfoVo.setPrivacyUseFlag(1);
        //if (staffPrivacyUseLogDTO != null) {
        //    wellDoneLoginInfoVo.setPrivacyUseFlag(staffPrivacyUseLogDTO.getPrivacyUseFlag());
        //}
        return resultResponse(HaoBanErrCode.ERR_1, wellDoneLoginInfoVo);
    }


    /**
     * 好办小程序-登陆
     *
     * @param staffLoginQO 员工登录:
     * @return {@link HaobanResponse }
     * @author mozhu
     * @date 2021-12-14 20:18:12
     */
    @RequestMapping(value = "welldone-login", method = RequestMethod.POST)
    @IgnoreLogin
    public RestResponse<String> welldoneLogin(@RequestBody @Valid StaffLoginQO staffLoginQO, HttpServletRequest request) {
        StaffDTO staffDTO = staffApiService.selectById(staffLoginQO.getStaffId());
        if (staffDTO == null) {
            return RestResponse.failure(String.valueOf(HaoBanErrCode.ERR_6.getCode()), HaoBanErrCode.ERR_6.getMsg());
        }
        //登录和刷新token兼容
        String headerToken = AuthRequestWellDoneUtil.getHeaderToken();
        if (StringUtils.isNotBlank(headerToken)) {
            String refreshToken = AuthRequestWellDoneUtil.refreshToken();
            if (StringUtils.isBlank(refreshToken)) {
                return RestResponse.failure(String.valueOf(HaoBanErrCode.ERR_30010.getCode()), HaoBanErrCode.ERR_30010.getMsg());
            }
            return RestResponse.successResult(refreshToken);
        }
        String staffId = staffDTO.getStaffId();
        Map<String, String> map = new HashMap<>();
        map.put("staffId", staffId);
        map.put("wxEnterpriseId", staffDTO.getWxEnterpriseId());
        map.put("phoneNumber", staffDTO.getPhoneNumber());
        String token = JwtUtil.genToken(map);
        AuthRequestWellDoneUtil.setAppLoginUser(staffId, token);
        StaffLoginDTO staffLoginDTO = new StaffLoginDTO();
        BeanUtils.copyProperties(staffLoginQO, staffLoginDTO);
        staffLoginDTO.setStaffIp(IPAddressUtil.getIpAddress(request));
        staffDTO.setWxEnterpriseId(staffDTO.getWxEnterpriseId());
        staffDTO.setPhoneNumber(staffDTO.getPhoneNumber());
        staffDTO.setWxUserId(staffDTO.getWxUserId());
        wxEnterpriseApiService.wellDoneLogin(staffLoginDTO);
        return RestResponse.successResult(token);
    }

    /**
     * 令牌token刷新
     *
     * @return {@link HaobanResponse }
     * @author mozhu
     * @date 2021-12-10 18:34:12
     */
    @RequestMapping(value = "welldone-token-refresh", method = RequestMethod.GET)
    public HaobanResponse welldoneTokenRefresh() {
        String token = AuthRequestWellDoneUtil.refreshToken();
        if (StringUtils.isBlank(token)) {
            return resultResponse(HaoBanErrCode.ERR_30010);
        }
        return resultResponse(HaoBanErrCode.ERR_1, token);
    }

    /**
     * 好办小程序-登出
     *
     * @return
     */
    @RequestMapping(value = "welldone-loginout", method = RequestMethod.GET)
    @IgnoreLogin
    public HaobanResponse welldoneLoginOut() {
        AuthRequestWellDoneUtil.delToken();
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 会员小程序-获取登录用户信息
     *
     * @param qo
     * @return
     */
    @RequestMapping("get-user-by-member-code")
    @IgnoreLogin
    public HaobanResponse getInfoByMemberCode(GetUserByMemberCodeQo qo) {
        WxEnterpriseDTO enterpriseDetailDTO = null;
        if (StringUtils.isNotBlank(qo.getCorpId())) {
            enterpriseDetailDTO = wxEnterpriseApiService.getEnterpriseBycorpId(qo.getCorpId());
            if (null == enterpriseDetailDTO) {
                logger.info("改企业没有关系好办1:{}", qo.getGicEnterpriseId());
                return resultResponse(HaoBanErrCode.ERR_500001);
            }
        } else {
            List<WxEnterpriseDTO> wxEnterpriseDTOS = wxEnterpriseRelatedApiService.listWxEnterpriseByEid(qo.getGicEnterpriseId());
            if (CollectionUtils.isEmpty(wxEnterpriseDTOS)) {
                logger.info("改企业没有关系好办:{}", qo.getGicEnterpriseId());
                return resultResponse(HaoBanErrCode.ERR_500001);
            }
            enterpriseDetailDTO = wxEnterpriseDTOS.get(0);
        }
        WxEnterpriseDTO enterpriseDTO = wxEnterpriseApiService.getOne(enterpriseDetailDTO.getWxEnterpriseId());
        if (enterpriseDTO == null) {
            logger.info("该企业不存在:{}", qo.getGicEnterpriseId());
            return resultResponse(HaoBanErrCode.ERR_400002);
        }
        MemberLoginQo user = new MemberLoginQo();
        if (StringUtils.isBlank(qo.getUserId())) {
            user = getUserByCode(enterpriseDTO, qo.getCode(), qo.getGicEnterpriseId());
            if (user == null) {
                return resultResponse(HaoBanErrCode.ERR_6);
            }
            WxApplicationDTO wxApplicationDTO = wxApplicationApiService.selectByCorpid(user.getCorpId());
            WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxApplicationDTO.getWxEnterpriseId());
            if (wxEnterpriseDTO == null) {
                logger.info("该企业不存在:{}", qo.getGicEnterpriseId());
                return resultResponse(HaoBanErrCode.ERR_400002);
            }
            if (!wxEnterpriseDTO.getCorpid().equals(enterpriseDTO.getCorpid())) {
                return resultResponse(HaoBanErrCode.ERR_500003, wxEnterpriseDTO.getCorpid());
            }
        } else {
            user.setUserId(qo.getUserId());
        }

        StaffDTO staffDTO = staffApiService.selectByUserIdAndEnterpriseId(user.getUserId(), enterpriseDTO.getWxEnterpriseId());
        if (staffDTO == null) {
            logger.info("成员不存在");
            return resultResponse(HaoBanErrCode.ERR_6);
        }
        //保存关联关系
        StaffOpenRelatedDTO relatedDTO = new StaffOpenRelatedDTO();
        relatedDTO.setMemberId(qo.getMemberId());
        relatedDTO.setOpenId(qo.getOpenid());
        relatedDTO.setStaffId(staffDTO.getStaffId());
        relatedDTO.setWxEnterpriseId(enterpriseDTO.getWxEnterpriseId());
        relatedDTO.setGicEnterpriseId(qo.getGicEnterpriseId());
        staffMemberRelationApiService.bindMemberAndStaff(relatedDTO);

        //补充字段赋值
        user.setStaffId(staffDTO.getStaffId());
        user.setWxEnterpriseId(staffDTO.getWxEnterpriseId());
        user.setStaffName(staffDTO.getStaffName());
        user.setPhoneNumber(staffDTO.getPhoneNumber());
        user.setNationCode(staffDTO.getNationCode());
        user.setWxUserId(staffDTO.getWxUserId());

        List<StaffClerkInfoDTO> result = staffClerkRelationApiService.listBindDetailByStaffId(staffDTO.getStaffId());

        List<StoreMemberVO> retList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(result)) {
            result.stream().filter(staffClerkInfoDTO -> {
                return staffClerkInfoDTO.getEnterpriseId().equals(qo.getGicEnterpriseId());
            }).forEach(dto -> {
                StoreMemberVO VO = new StoreMemberVO();
                StoreDTO store = storeService.getStore(dto.getStoreId());
                ClerkDTO clerkDTO = clerkService.getClerkByClerkId(dto.getClerkId());
                if (null != store && clerkDTO != null) {
                    VO.setUserId(staffDTO.getWxUserId());
                    VO.setStoreId(dto.getStoreId());//为空，则无该值
                    VO.setStoreName(dto.getStoreName());
                    VO.setWxEnterpriseId(dto.getWxEnterpriseId());
                    VO.setEnterpriseId(store.getEnterpriseId());
                    VO.setStoreImg(store.getImageUrl());//先置空
                    VO.setBindFlag(1);
                    VO.setClerkCode(clerkDTO.getClerkCode());
                    VO.setClerkType(clerkDTO.getClerkType());
                    VO.setStaffId(staffDTO.getStaffId());
                    VO.setClerkId(clerkDTO.getClerkId());
                    retList.add(VO);
                }

            });
            user.setStoreList(retList);
        }
        return resultResponse(HaoBanErrCode.ERR_1, user);
    }

    private MemberLoginQo getUserByCode(WxEnterpriseDTO enterpriseDTO, String code, String enterpriseId) {
        SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.MEMBER_WAPP.getVal(), enterpriseId);
        if (null == secretSetting || StringUtils.isBlank(secretSetting.getSecretVal())) {
            logger.info("MEMBER_WAPPsecret配置没1ss有配置");
            return null;
        }
        String userDTOJson = qywxUserApiService.getUserByCode(enterpriseDTO.getCorpid(), secretSetting.getSecretVal(), code);
        logger.info("会员微信登录信息=============>{}", userDTOJson);
        return JSON.parseObject(userDTOJson, MemberLoginQo.class);
    }

    /**
     * 好办点击进来日志表
     *
     * @param userLoginLogDTO
     * @return
     */
    @RequestMapping("save-user-login-log")
    public HaobanResponse saveUserLoginLog(UserLoginLogDTO userLoginLogDTO) {
        staffApiService.insertUserLoginLog(userLoginLogDTO);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("get-member-store")
    public HaobanResponse getMemberStore(String externalUserid, String wxUserId, String storeId) {
        if (StringUtils.isAnyBlank(externalUserid, wxUserId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        List<MemberStoreDTO> result = memberUnionidRelatedApiService.listByExTernalUseridAndWxUserId(externalUserid, wxUserId, storeId);
        return resultResponse(HaoBanErrCode.ERR_1, result);
    }

    @RequestMapping("get-staff-info")
    public HaobanResponse getStaffInfo(String phoneNumber) {
        if (StringUtils.isBlank(phoneNumber)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        List<StaffDTO> list = staffApiService.listByPhoneNumber(phoneNumber);
        if (CollectionUtil.isNotEmpty(list)) {
            List<AppStaffVo> staffs = EntityUtil.changeEntityListByJSON(AppStaffVo.class, list);
            Set<String> wxEnterpriseIds = list.stream().map(s -> s.getWxEnterpriseId()).collect(Collectors.toSet());
            List<YwWxEnterpriseDTO> enterprises = wxEnterpriseApiService.listByIds(wxEnterpriseIds);
            Map<String, YwWxEnterpriseDTO> map = com.gic.commons.util.CollectionUtil.toMap(enterprises, "wxEnterpriseId");
            for (AppStaffVo vo : staffs) {
                String wxEnterpriseId = vo.getWxEnterpriseId();
                String corpName = map.get(wxEnterpriseId) == null ? "" : map.get(wxEnterpriseId).getCorpName();
                vo.setWxEnterpriseName(corpName);
            }
            return resultResponse(HaoBanErrCode.ERR_1, staffs);
        }
        return resultResponse(HaoBanErrCode.ERR_1, new ArrayList<>());
    }

    /**
     * 获取绑定信息(会员小程序用)
     *
     * @param externalUserid
     * @param wxUserId
     * @param wxEnterpriseId
     * @return
     */
    @RequestMapping("get-send-message-info")
    @IgnoreLogin
    public HaobanResponse getMessageMember(String externalUserid, String wxUserId, String wxEnterpriseId, String gicEnterpriseId) {
        if (StringUtils.isAnyBlank(externalUserid, wxUserId, wxEnterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        WxEnterpriseRelationDetailDTO enterpriseBindInfo = wxEnterpriseRelatedApiService.getEnterpriseBindInfo(wxEnterpriseId, gicEnterpriseId);
        if (null == enterpriseBindInfo) {
            logger.info("企业不存在没有绑定");
            return resultResponse(HaoBanErrCode.ERR_400002);
        }
        MemberUnionidRelatedDTO memberUnionidRelatedDTO = memberUnionidRelatedApiService.getBySelfExternalUserId(wxEnterpriseId, externalUserid);
        if (null == memberUnionidRelatedDTO) {
            logger.info("请刷新好友");
            return resultResponse(HaoBanErrCode.ERR_11123);
        }
        ExternalClerkRelatedDTO clerkRelatedDTO = externalClerkRelatedApiService.getExternalClerkByWxUserIdAncExternalUserId(wxEnterpriseId, wxUserId, gicEnterpriseId, memberUnionidRelatedDTO.getExternalUserid());
        if (clerkRelatedDTO == null) {
            logger.info("该企业没有对应需要发送好友关系的信息");
            return resultResponse(HaoBanErrCode.ERR_11123);
        }
        MiniprogramSettingDTO settingDTO = wxEnterpriseApiService.getMiniprogramSettingByEid(wxEnterpriseId, gicEnterpriseId);
        if (null == settingDTO) {
            logger.info("该企业没有对应需要发送的小程序信息");
            return resultResponse(HaoBanErrCode.ERR_11124);
        }
        EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(gicEnterpriseId);
        MemberSendMessageVo memberSendMessageVo = new MemberSendMessageVo();
        String relationKey = clerkRelatedDTO.getRelationKey();
        memberSendMessageVo.setRelationKey(StringUtils.isNotBlank(relationKey) ? relationKey : "-1");
        memberSendMessageVo.setImageUrl(settingDTO.getImageUrl());
        memberSendMessageVo.setMediaId(settingDTO.getMediaId());
        memberSendMessageVo.setSelfExteralUserId(externalUserid);
        memberSendMessageVo.setTitle(settingDTO.getTitle());
        memberSendMessageVo.setWxaHandUrl(enterpriseDTO.getMallMiniprogramLogoUrl());
        memberSendMessageVo.setBindFlag(clerkRelatedDTO.getStatusFlag() == 1 ? 1 : 0);
        memberSendMessageVo.setWxaName(settingDTO.getMiniprogramName());
        return resultResponse(HaoBanErrCode.ERR_1, memberSendMessageVo);
    }

    /**
     * 绑定好友（会员小程序调用）
     *
     * @param memberId
     * @param enterpriseId
     * @param unionid
     * @param relationKey
     * @return
     */
    @RequestMapping("member-bind-friend")
    @IgnoreLogin
    public HaobanResponse memberBindFriend(String memberId, String enterpriseId, String unionid, String relationKey) {
        if (StringUtils.isAnyBlank(memberId, enterpriseId, relationKey)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        logger.info("会员绑定好友，memberId:{},enterpriseId:{},unionid:{},relationKey:{}", enterpriseId, memberId, unionid, relationKey);
        memberOpenCardBusinessService.memberClerkProcessByQywxFriend(enterpriseId, memberId, unionid, relationKey);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

}
